LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

-- This entity is a register with multiple inputs.
-- When address is 00, the register maintains its value.
-- When address is 01, the register increments itself.
-- When address is 10, the register loads the value on Data1.
-- When address is 11, the register loads the value on Data2.
ENTITY lightreg IS
	PORT(
		Clk, Rst: 		IN	STD_LOGIC;								-- reset, clock
		Sel: 	IN STD_LOGIC_VECTOR(1 downto 0);			-- action selector
		Inst: IN	STD_LOGIC_VECTOR(4 downto 0);	-- alternative load inputs
		Acc: IN STD_LOGIC_VECTOR(7 DOWNTO 0);   
		Q : OUT	STD_LOGIC_VECTOR(7 downto 0));					-- output
END lightreg;

ARCHITECTURE behavioral OF lightreg IS
	SIGNAL Qreg: UNSIGNED (7 downto 0);	-- internal register representation, stores output.
BEGIN
	action:	PROCESS(Clk, Rst) BEGIN
		if Rst = '1' then Qreg <= "00000000";
		elsif clk = '1' and clk'event then
			case Sel is	-- if there is a rising edge, then load from the selected source
			when "00" => if Inst(4) = '0' then 
							Qreg(3 downto 0) <= UNSIGNED(Inst(3 downto 0)); 
							Qreg(7 downto 4) <= Qreg(7 downto 4);-- Load from the instruction register
						else Qreg(3 downto 0) <= Qreg(3 downto 0);
							Qreg(7 downto 4) <= UNSIGNED(Inst(3 downto 0));
						end if;
			when "01" => Qreg <= UNSIGNED(Acc);
			when "11" => Qreg(0) <= not Qreg(0);
							Qreg(1) <= not Qreg(1);
							Qreg(2) <= not Qreg(2);
							Qreg(3) <= not Qreg(3);
							Qreg(4) <= not Qreg(4);
							Qreg(5) <= not Qreg(5);
							Qreg(6) <= not Qreg(6);
							Qreg(7) <= not Qreg(7);
			when others => Qreg <= Qreg;			-- maintain value on all other inputs
			end case;								-- Note: since std_logic is a multi-
		end if;										-- value logic, this keeps the synthesis simple
	END PROCESS action;

	Q <= STD_LOGIC_VECTOR(Qreg);	-- write the register value to the output
		   							-- you could put an output enable switch here as well.
END behavioral;




